/***************************************************************************/
/* CDEMO - DCX-AT200 'C' Interface Demo Program                            */
/*  Precision Micro Control Corp. 1988-1994                                */
/*  Revision: 1.1a                                                         */
/*  Date Last Modified: 9/6/94                                             */
/*  Programmer: Bruce Robinson                                             */
/*  Compiler: Microsoft C                                                  */
/*                                                                         */
/* Abstract: This file contains a program for demonstrating the use of     */
/* the DCX high level language interface for the C programming language.   */
/* The code in this module must be compiled and the linked with the DCX    */
/* interface code.                                                         */
/*                                                                         */
/* In order to build the program under Microsoft C, use the following      */
/* command line:                                                           */
/*   CL CDEMO.C DCX2CIFC.C                                                 */
/*                                                                         */
/* Revision History                                                        */
/*  1.0a - Initial release (created from DCX series 100 version: cdemo)    */
/*  1.1a - Modified program to use DCX 'C' interface macros, see           */
/*          DCX2CMAC.H file                                                */
/*                                                                         */
/***************************************************************************/

#include "dcx2cifc.h"	/* DCX-AT200 'C' interface library include file */
#include "dcx2cmac.h"	/* DCX-AT200 'C' macros */

#define ESC 0x1b			/* Escape key code for exiting program */

main()
{
	char ky,axison;
	int board,axis;
	double position,optimal,target;
	double distance;

	/* Put instructions on screen. */
	printf("DCX interface library demo - hit ESCAPE key to exit.\r\n");
	printf("Cursor control keys cause axis 1 to jog by 1000 steps if\r\n");
	printf("NUM LOCK is on, 1 step if it is off.  All other keys\r\n");
	printf("have no effect.\r\n");
	printf("Motor parameters should be set before this program is run!\r\n\n");

	board = 0;		/* DCX board number, must match rotary switch on DCX. */
	axis = 1;

	printf("AXIS %d: TRUE POSITION     DESIRED POSITION\
     TARGET POSITION\r\n",axis);

	/* Send commands to DCX board to turn axis 1 motor off, and set its
		position to zero. */
	motor_on(board,axis);
	define_home(board,axis,0);
	axison = FALSE;

	while(1)
	{
		/* Get motor's true, desired and target positions */
		position = get_position(board,axis);
		optimal = get_optimal(board,axis);
		target = get_target(board,axis);

		/* Display the positions */
		printf("         %+12.3f         %+12.3f        %+12.3f\r",
						position,optimal,target);

		if (kbhit())			/* Check for key press. */
		{
			if (!axison)			/* If motor is off, turn it on. */
			{
				motor_on(board,axis);
				axison = TRUE;
			}
			if((ky=getch()) == ESC)		/* Exit if ESCAPE key hit. */
				exit(0);
			else if(ky==0)			/* If keycode is zero, read it again. */
			{
				ky=getch();
									/* Set move distance in parameter array.*/
				if (ky==0x4B || ky==0x50)			/* Left or down arrow. */
					distance = -1.0;
				else if (ky==0x4D || ky==0x48)	/* Up or right arrow. */
					distance = 1.0;
				else distance = 0.0;
			}
			else if (ky=='4' || ky=='2')			/* Left or down arrow. */
				distance = -1000.0;
			else if (ky=='6' || ky=='8')			/* Up or right arrow. */
				distance = 1000.0;
			else
				distance = 0.0;

			/* Issue move relative command to DCX board. */
			move_relative(board,axis,distance);
		}
	}
}



