/*
	DCX2CIFC.H - DCX-AT200 'C' Interface Header File
   Copyright Precision Micro Control Corp. 1988-1994, All rights reserved
   Revision: (see DCX2CIFC.C)
   Date Last Modified: 9/6/94
   Programmer: Bruce Robinson

	This file contains header information for use of the DCX high level
	language interface for the C programming language. The code in this
	module can be included into the end users application programs.

	Revision History:  See DCX2CIFC.C

*/


#define FALSE   0
#define TRUE    1

#define MOTORS_PER_BOARD	6
#define MACROS_PER_BOARD	1024

/* Module ID codes */
#define MC200	0		/* Advanced Servo */
#define MC260	1		/* Advanced Stepper */
#define MC110  4		/* Servo with motor drive output */
#define MC120  4		/* Servo with PWM output */
#define MC100  5		/* Servo with analog signal output */
#define MC150  6		/* Stepper motor */
#define MC400  8		/* Digital I/O module */
#define MC500  12		/* Analog I/O module */
#define MF310  9		/* GPIB interface */
#define MF300  10		/* RS232 interface */
#define NOMOD  15		/* No module installed */

/* Axis values for parameters to dcxcmd() and dcxvcmd() functions. */ 
#define AXIS1	0x100
#define AXIS2	0x200
#define AXIS3	0x300
#define AXIS4	0x400
#define AXIS5	0x500
#define AXIS6	0x600

/* Analog output limit */
#define OUT_LIMIT 10.0

/* Modifier flags for axis bytes in the dcxcmd() and dcxvcmd() function
	parameters */
#define INDPARFLAG	0x8000	/* Indirect parameter flag */
#define NOPARFLAG		0x6000	/* No parameter flag */
#define FLTPARFLAG	0x4000	/* Float parameter flag */
#define DBLPARFLAG	0x2000	/* Double parameter flag */

/* Number of bytes in binary DCX commands (depends on type of parameter) */
#define CMDSZ		2	   /* no parameter */
#define LNGCMDSZ	6	   /* with long parameter */
#define FLTCMDSZ	6	   /* with float parameter */
#define DBLCMDSZ	10	   /* with double parameter */

/* Parameter values for Tell commands */
#define LNGRPY		0		/* Tell command will have long integer reply format */
#define DBLRPY		1		/* Tell command will have double reply format */
#define FLTRPY		2		/* Tell command will have float reply format */

/* Motor data offsets */
#define STAT		0x00	/* Motor Status */
#define POSCNT		0x04	/* Position Count (encoder or steps)*/
#define OPTCNT		0x08	/* Optimal Count (encoder or steps) */
#define IDXCNT 	0x0C	/* Index Counts (encoder or steps) */
#define AUXSTAT	0x10	/* Auxiliary Motor Status */
#define POSITION	0x14	/* Position in user units */ 
#define TARGET		0x1C	/* Target in user units */
#define OPTIMAL	0x24	/* Optimal position in user units*/
#define BRKPOS		0x2C	/* Break position in user units*/
#define DEADBAND	0x34	/* Deadband in user units */
#define MAXERROR	0x3C	/* Maximum following error in user units */
#define LOLIMIT	0x44	/* Low limit of movement in user units */
#define HILIMIT	0x4C	/* High limit of movement in user units */ 
#define SCALE		0x54	/* User Scale */
#define ZERO		0x5C	/* User Zero */
#define OFFSET		0x64	/* User Offset */
#define RATE		0x6C	/* User Rate conversion */
#define OUTCONST	0x74	/* User output Konstant */
#define PGMVEL		0x7C	/* Programmed velocity in user units */ 
#define PGMACC		0x84	/* Programmed acceleration in user units */ 
#define PGMDEC		0x8C	/* Programmed deceleration in user units */ 
#define MINVEL		0x94	/* Minimum velocity in user units */
#define CURVEL		0x9C	/* Current velocity in user units */
#define KVEL		0xA4	/* Velocity gain, in user distance units */
#define KACC		0xA8	/* Acceleration gain, in user distance units */
#define KDEC		0xAC	/* Deceleration gain, in user distance units */
#define VELOVRD	0xB0	/* Velocity override */
#define MAXTRQ		0xB4	/* Torque limit */
#define KPOS		0xB8	/* Proportional gain */
#define KDER		0xBC	/* Derivative gain */
#define KINT		0xC0	/* Integral gain */
#define ILIM		0xC4	/* Integration limit */
#define ADCIN1		0xC8	/* Analog input 1 on module */
#define ADCIN2		0xCC	/* Analog input 2 on module */
#define JGAIN		0xD0	/* Jog proportional Gain */
#define JOFFSET	0xD4	/* Jog Offset */
#define JBAND		0xD8	/* Jog deadBand */
#define JOGACC		0xDC	/* Jog acceleration in user units */ 
#define JOGMINVEL	0xE4	/* Jog minimum velocity in user units */
#define WAITSTOP	0xEC	/* Wait stopped timer */
#define WAITTRGT	0xEE	/* Wait target timer */
#define SFREQ		0xF0	/* Sampling frequency */
#define MASTER		0xF2	/* Set to axis that is master */
#define MODST		0xF4	/* Module status */
#define AXISNUM	0xF6	/* Axis number 1-6 */
#define MODULE		0xF8	/* Module position */
#define MODTYPE	0xFA	/* Module type */

/* Motor status bit definitions */
#define MTRBSY	0x1			/* BUSY (MOTOR DATA BEING UPDATAED) */
#define MTRON	0x2			/* MOTOR ON */
#define MTRAT	0x4			/* AT TARGET */
#define MTRTC	0x8			/* TRAJECTORY COMPLETE */
#define MTRDI	0x10			/* DIRECTION (0=POSITIVE, 1=NEGATIVE) */
#define MTRJOG	0x20			/* JOGGING IS ENABLED */
#define MTRHMD	0x40			/* MOTOR HOMED */
#define MTRERR	0x80			/* MOTOR ERROR */
#define MTRLFI	0x100			/* LOOK FOR INDEX */
#define MTRLFE	0x200			/* LOOK FOR EDGE */
#define MTRBRK	0x1000		/* BREAKPOINT REACHED */
#define MTREFE	0x2000		/* EXCEEDED MAX. FOLLOWING ERROR */
#define MTRAFE	0x4000		/* AMPLIFIER FAULT ENABLED */
#define MTRAFT	0x8000		/* AMPLIFIER FAULT TRIPPED */
#define MTRLPE	0x10000		/* HARD LIMIT POSITIVE ENABLED */
#define MTRLPT	0x20000		/* HARD LIMIT POSITIVE TRIPPED */
#define MTRLNE	0x40000		/* HARD LIMIT NEGATIVE ENABLED */
#define MTRLNT	0x80000		/* HARD LIMIT NEGATIVE TRIPPED */
#define MTRLHE	0x100000		/* SOFT POSITION LIMIT HIGH ENABLED */
#define MTRLHT	0x200000		/* SOFT POSITION LIMIT HIGH TRIPPED */
#define MTRLLE	0x400000		/* SOFT POSITION LIMIT LOW ENABLED */
#define MTRLLT	0x800000		/* SOFT POSITION LIMIT LOW TRIPPED */
#define MTRIDX	0x1000000	/* ENCODER INDEX / STEPPER HOME */
#define MTRCRS	0x2000000	/* COARSE HOME */
#define MTREE	0x4000000	/* AMPLIFIER FAULT */
#define MTRRSV	0x8000000	/* AUXILIARY ENCODER INDEX */
#define MTRLMR	0x10000000	/* LIMIT POSITIVE INPUT ACTIVE */
#define MTRLML	0x20000000	/* LIMIT NEGATIVE INPUT ACTIVE */
#define MTRJGR	0x40000000	/* USER INPUT 1 */
#define MTRJGL	0x80000000	/* USER INPUT 2 */


/* DCX Command Codes */
/* Parameter Setup Commands */
#define	AG		226		/* set Acceleration Feed-Forward Gain */
#define	AH		234		/* Auxiliary encoder define Home */
#define	BR		30			/* Baud Rate */
#define	CF		31			/* Channel ofF */
#define	CH		66			/* Channel High true */
#define	CI		32			/* Channel In */
#define	CL		67			/* Channel Low true */
#define	CN		33			/* channel oN */
#define	CT		34			/* Channel ouT */
#define	DB		118		/* set DeadBand */
#define	DG		227		/* set Deceleration Feed-Forward Gain */
#define	DH		35			/* Define Home */
#define	DI		36			/* DIrection */
#define	DS		117		/* Deceleration Set */
#define	DT		197		/* at Target Delay */
#define	EF		37			/* Echo ofF */
#define	EN		38			/* Echo oN */
#define	FC		64			/* Full Current */
#define	FF		51			/* Fail input ofF */
#define	FN		50			/* Fail input oN */
#define	FR		39			/* set derivative sampling period */
#define	HC		65			/* Half Current */
#define	HF		48			/* RS232 module Handshake ofF */
#define	HL		211		/* set motion High Limit */
#define	HN		49			/* RS232 module Handshake oN */
#define	HS		232		/* High Speed */
#define	IL		40			/* set Integration Limit */
#define	JA		56			/* Jog Acceleration */
#define	JB		223		/* Jog Deadband */
#define	JF		58			/* Jog ofF */
#define	JG		220		/* Jog Gain */
#define	JN		57			/* Jog oN */
#define	JO		222		/* Jog Offset */
#define	JV		55			/* Jog Velocity */
#define	LF		54			/* Limit switch ofF */
#define	LL		210		/* set motion Low Limit */
#define	LM		52			/* Limit switch Mode */
#define	LN		53			/* Limit switch oN */
#define	LS		230		/* Low Speed */
#define	MS		231		/* Medium Speed */
#define	MV		196		/* set Minimum Velocity */
#define	OM		216		/* set Output Mode */
#define	PC		128		/* Prompt Character */
#define	PH		115		/* Set Phasing */
#define	PS		238		/* Profile S-curve */
#define	PT		239		/* Profile Trapezoidal */
#define	RC		41			/* Reset Counter */
#define	RT		42			/* ReseT system */
#define	SA		43			/* Set Acceleration */
#define	SD		44			/* Set Derivative gain */
#define	SE		25			/* Stop on Error */
#define	SF		62			/* Step Full */
#define	SG		45			/* Set Gain */
#define	SH		63			/* Step Half */
#define	SI		46			/* Set Integral gain */
#define	SQ		116		/* Set torque */
#define	SS		29			/* Set Slave ratio */
#define	SV		47			/* Set Velocity */
#define	UK		215		/* set User output Konstant */
#define	UO		179		/* set User Offset */
#define	UR		177		/* set User Rate conversion */
#define	US		175		/* set User Scale */
#define	UT		178		/* set User Time conversion */
#define	UZ		176		/* set User Zero */
#define	VA		173		/* Vector Acceleration */
#define	VD		174		/* Vector Deceleration */
#define	VG		119		/* set Velocity Gain */
#define	VO		224		/* set Velocity Override */
#define	VV		172		/* Vector Velocity */
#define	VA		173		/* Vector Acceleration */
#define	VD		174		/* Vector Deceleration */
#define	XF		126		/* Set XOF character */
#define	XN		125		/* Set XON character */

/* Motion Commands */
#define	AB		10			/* ABort */
#define	AF		235		/* Auxiliary encoder Find index */
#define	CA		180		/* arc Center Absolute */
#define	CD		193		/* Contour increment Distance */
#define	CM		27			/* Set Contouring Mode */
#define	CP		192		/* Contour Path */
#define	CR		181		/* arc Center Relative */
#define	FE		11			/* Find Edge */
#define	FI		12			/* Find Index */
#define	GH		13			/* Go Home */
#define	GM		 8			/* Gain Mode */
#define	GO		14			/* GO */
#define	HO		15			/* HOme */
#define	IA		93			/* Index Arm */
#define	LP		112		/* Learn Position */
#define	LT		113		/* Learn Target */
#define	MA		16			/* Move Absolute */
#define	MF		17			/* Motor ofF */
#define	MN		19			/* Motor oN */
#define	MP		20			/* Move to Point */
#define	MR		21			/* Move Relative */
#define	NS		171		/* No Syncronization */
#define	PM		23			/* Position Mode */
#define	QM		 9			/* Torque Mode */
#define  SM    26       /* Set Master */
#define	SN		170		/* Syncroniztion oN */
#define	ST		22			/* STop */
#define	VM		24			/* Velocity Mode */

/* Reporting Commands */ 
#define	AT		233		/* Auxiliary encoder Tell position */
#define	AZ		236		/* Auxiliary encoder tell Index */
#define	CC		70			/* Current Count */
#define	CS		71			/* CheckSum */
#define	HE		72			/* HElp */
#define	TA		73			/* Tell A/d */
#define	TB		91			/* Tell Breakpoint Position */
#define	TC		74			/* Tell Channel */
#define	TD		75			/* Tell Derivative Gain */
#define	TF		77			/* Tell Following error */
#define	TG		78			/* Tell Position Gain */
#define	TI		79			/* Tell Integral Gain */
#define	TK		92			/* Tell velocity Konstant */
#define	TL		80			/* Tell Integration Limit */
#define	TO		89			/* Tell Optimal */
#define	TP		82			/* Tell Position */
#define	TQ		209		/* Tell torQue */
#define	TS		83			/* Tell Status */
#define	TT		84			/* Tell Target */
#define	TV		85			/* Tell Velocity */
#define	VE		86			/* tell VErsion */
#define	TX		88			/* Tell Contouring Count */
#define	TZ		90			/* Tell Index */
	
/* Register Commands */
#define	AA		133		/* Accumulator Add */
#define	AC		140		/* Accumulator Complement */
#define	AD		136		/* Accumulator Divide */
#define	AE		143		/* Accumulator Eor */
#define	AL		130		/* Load accumulator with constanat */
#define	AM		135		/* Accumulator Multiply */
#define	AN		141		/* Accumulator aNd */
#define	AO		142		/* Accumulator Or */
#define	AR		132		/* Copy accumulator to register */
#define	AS		134		/* Accumulator Subtract */
#define	AV		139		/* Accumulator eValuate */
#define	GA		248		/* Get Analog input reading into accumulator */
#define	GX		247		/* Get auXiliary encoder position into accumulator */
#define	OA		249		/* Output Analog value from accumulator */
#define	RA		131		/* Accumulator load with register */
#define	RB		150		/* Read accumulator Byte */
#define	RD		147		/* Read accumulator double (64 bit floating point) */
#define	RL		152		/* Read accumulator Long */
#define	RV		146		/* Read accumulator float (32 bit floating point) */
#define	RW		151		/* Read accumulator Word */
#define	SL		144		/* accumulator Shift Left */
#define	SR		145		/* accumulator Shift Right */
#define	TR		87			/* Tell Register */
#define	WB		153		/* Write accumulator Byte */
#define	WD		149		/* Write accumulator double (64 bit floating point) */
#define	WL		155		/* Write accumulator Long */
#define	WV		148		/* Write accumulator float (32 bit floating point) */
#define	WW		154		/* Write accumulator Word */

/* Macro Commands */
#define	ET		251		/* Escape Task */
#define	GT		250		/* Generate Task */
#define	MC		2			/* Macro Command */
#define	MD		3			/* Macro Definition */
#define	MJ		5			/* Macro Jump */
#define	RM		4			/* Reset Macros */
#define	TM		81			/* Tell Macros */

/* Sequence Commands */
#define	DF		107		/* Do if channel ofF */
#define	DN		106		/* Do if channel oN */
#define	IB		165		/* If accumulator Below (less then) */
#define	IC		161		/* If bit Clear in accumulator */
#define	IE		162		/* If accumulator Equal */
#define	IF		109		/* If channel ofF then, else skip two */
#define	IG		164		/* If accumulator Greater than */
#define	II		68			/* If Intermediate */
#define	IN		108		/* If channel oN then, else skip two */
#define  IP    96       /* Interrupt on absolute Position */
#define  IR    97       /* Interrupt on Relative position */
#define	IS		160		/* If bit Set in accumulator */
#define	IU		163		/* If accumulator Unequal */
#define	JP		6			/* Jump to Command Absolute */
#define	JR		7			/* Jump to command Relative */
#define  RP    100      /* RePeat */
#define	WA		101		/* WAit (time) */
#define	WE		102		/* Wait Edge */
#define	WF		103		/* Wait for Channel ofF */
#define	WI		94			/* Wait for Index */
#define	WN		104		/* Wait for Channel oN */
#define  WP    98       /* Wait for absolute Position */
#define  WR    99       /* Wait for Relative position */
#define	WS		105		/* Wait for Stop */
#define	WT		198		/* Wait for Target */

/* Miscellaneous Commands */
#define	BK		121		/* Break */
#define	DM		60			/* Decimal Mode */
#define	HM		61			/* Hex Mode */
#define	NO		120		/* No Operation */


/* Contour Path Modes */
#define	USER_DEF	0
#define	LINE		1
#define	ARC_CW	2
#define	ARC_CCW	3


/* DCX function return values. */
#define DCXOK	0


/* DCX dual port memory equates. */
#define DCXSEG 0xD000		/* Memory sement where DCX dual port ram is. */
#define DATAOUT 0x804      /* Memory offset for data out mailbox. */
#define DATAIN 0x800       /* Memory offset for data in mailbox. */
#define CMDSTAT 0x808		/* Memory offset for command status. */
#define MODLOC 0x80A			/* Memory offset for motor module locations. */
#define MODTYP 0x812			/* Memory offset for array of module types. */
#define CMDCNT 0x900			/* Memory offset for command count. */
#define CMDBLK 0x901			/* Memory offset for command block. */
#define RPYCNT 0xA00			/* Memory offset for reply count. */
#define RPYBLK 0xA01			/* Memory offset for reply block. */


/* Structures for declaring command buffers (note the different
	parameter types). */
struct cmdfmt
{
	int axis;
	long param;
};

/* Long parameter format */
struct cmdlngfmt
{
	int axis;
	long param;
};

/* Float parameter format */
struct cmdfltfmt
{
	int axis;
	float param;
};

/* Double parameter format */
struct cmddblfmt
{
	int axis;
	double param;
};


/* Structures for declaring reply buffers (note the different
	reply types) */
struct rpyfmt
{
	int axis;
	long val;
};

/* Long reply format */
struct rpylngfmt
{
	int axis;
	long val;
};

/* Float reply format */
struct rpyfltfmt
{
	int axis;
	float val;
};

/* Double reply format */
struct rpydblfmt
{
	int axis;
	double val;
};


/* External declarations of buffers for receiving replies. These buffers
	are used in DCX2CMAC.H */
extern struct rpylngfmt lngrpybuf[MOTORS_PER_BOARD];
extern struct rpyfltfmt fltrpybuf[MOTORS_PER_BOARD];
extern struct rpydblfmt dblrpybuf[MOTORS_PER_BOARD];


/* Interface function prototypes */
int dcxcmd(int,int,int,...);

int dcxvcmd(int,int,int,...);

int dcxrpy(int,int,int far *);

#ifndef DCXLIB

extern int dcxrdy(int);

#endif


/* End of DCXCIFC.H */
