/*
    DCX-AT200 Loader Program
    Revision: 1.5a
    Date Last Modified: 5/7/96
    Programmer: Bruce Robinson                                                       
    Precision MicroControl Corporation
    Copyright (c) 1996, All rights reserved                                       

    Abstract:
       This program is used for loading a file on the DCX-AT200.

    Example: DCX2LOAD textfile

	 There are a number of options the user can specify when invoking this
    program. The following option specifiers are supported:

		/Bn		Causes program to access board n (default = 0).

		/Ctext	Causes program to issue the text to the controller as a
					 command prior to loading the file. Text can be in double
					 quotes or not, with no intervening spaces after /C.


    Example: DCX2LOAD /C"LO1" textfile
*/

/* To build program under Microsoft C use:  CL DCX2LOAD.C	*/

/*
	Revision History
	1.5a	Initial release. Created from DCX2CTRL program, revision 1.5a.
*/

#include <dos.h>
#include <stdio.h>
#include <string.h>

#define SUB 0x1A				/* Ctrl-Z for End Of File */
#define ESC 0x1B				/* ESCape code */

#define DATAIN 0x800			/* Memory offset for data input mailbox. */
#define DATAOUT 0x804		/* Memory offset for data output mailbox. */
#define CMDSTAT 0x808		/* Memory offset for command status. */

unsigned int dcxseg = 0xD000;	/* Memory sement where DCX dual port ram is. */

unsigned char byt;			/* Byte used during reads to far memory. */
unsigned char far *bytptr;		/* Note FAR prefix of this pointer. */

char cmdline[256];


main(argc,argv)
int argc;
char *argv[];
{
	int argi;			/* Command line argument index. */
	char option;		/* Command line options. */
	char chrout = 0;		/* Last character from file. */
	char chrin;			/* Last character received from DCX output mailbox. */
	int idx = -1;
	int filenum;
	int bufcnt;
	FILE *loadfile = 0;
	char bdnumchar;

	printf("DCX-AT200 Loader Program   Revision 1.5a\r\n");
	printf("Copyright (c) Precision MicroControl Corporation 1996\r\n");
	printf("All rights reserved\r\n");

	bytptr = &byt;			/* Initialize far pointer. */

	cmdline[0] = '\0';

												/* Parse command line */
	for (argi = 1; argi < argc; argi++)
	{
		if (*argv[argi] == '/')			/* Test for command option having '/' */
		{
			option = toupper(*(argv[argi]+1));
											/* 'Bn' option will cause program to access
												selected DCX board. */
			if (option=='B')
			{
				bdnumchar = toupper(*(argv[argi]+2));
				if ((bdnumchar >= '0') && (bdnumchar <= '9'))
					dcxseg += ((bdnumchar - '0') << 8);
				else if ((bdnumchar >= 'A') && (bdnumchar <= 'F'))
					dcxseg += ((bdnumchar - 'A') << 8);
				else
				{
					printf("bad board number\r\n");
					exit(0);
				}
				printf("board number %c\r\n",bdnumchar);
			}
											/* 'Ctext' option will cause program to
												issue the text to the controller prior
												to sending the file. */
			if (option=='C')
			{
											/* Create initial command string */
				strcpy(cmdline,argv[argi]+2);
				bufcnt = strlen(cmdline);
				cmdline[bufcnt++] = '\r';	/* Terminate the string with a return */
				cmdline[bufcnt++] = '\0';
				idx = 0;
				printf("command: %s\r\n",cmdline);
			}
		}
		else						/* If not command option, should be file name. */
		{
									/* Try to open file for reading. */
			if (loadfile = fopen(argv[argi],"r"))
				printf("load file: %s\r\n", argv[argi]);
			else
			{
				/* If unable to open file, print message and exit. */
				printf("can't open: %s\r\n", argv[argi]);
				exit(0);
			}
		}
	}

	/* Main loop. Breaking out of this loop requires that the file has been
		sent, the controller has terminated any command, and the
		prompt character (if any) sitting in the DCX output mailbox is
		received */
	while (loadfile || getdcxbyt(CMDSTAT) || getdcxbyt(DATAOUT))
	{
		/* If contents of chrout is 0, get next character from initial
			command string or a character from the load file. */
		if (!chrout)
		{
							/* Try getting character from initial command string. */
			if (chrout = cmdline[idx])
			{
				idx++;
			}

			else if (loadfile && (getdcxbyt(CMDSTAT) & 0x80))
			{
							/* Get next character from load file. */
				if ((chrout = fgetc(loadfile)) == EOF)
				{
					if (fclose(loadfile) != 0)
						printf("error closing file\r\n");
					loadfile = 0;			/* Set file pointer to zero to indicate
													download has completed */
					chrout = SUB;				/* Send ctrl-Z to mark end of file */
				}

				/*  Check for an escape sequence (found in HPGL plotter files)  */
				else if (chrout==ESC)
				{
					chrout = fgetc(loadfile);
					if (chrout=='.')
					{
						char escape_code,escape_terminator;

						/*  Read the first character after the escape sequence. */
						escape_code = chrout = fgetc(loadfile);
						switch(escape_code){
							case '@':
							case 'H':
							case 'I':
							case 'M':
							case 'N':
								escape_terminator = ':';
								break;
							default:
								escape_terminator = 0;
								break;
						}

						/* A non-zero escape terminator implies we need to read the
							text following the escape sequence up to the terminator */
						if (escape_terminator)
						{
							/*  Read the text from the file until the terminator of the
								sequence	is reached. */
							chrout = fgetc(loadfile);
							while ((chrout!=escape_terminator) && (chrout!=EOF))
							{
								chrout = fgetc(loadfile);
							}

							/* Check for unexpected end of file. */
							if (chrout == EOF)
							{
								printf("ERROR: escape sequence has no end\r\n");
								exit(1);
							}
						}
					}
					else
					{
						/*  If a '.' does not follow the escape character, we have
							a problem.   */
						printf("ERROR: invalid escape sequence\r\n");
						exit(1);
					}
					chrout = 0;			/* Set chrout back to zero to cause the character
											following the ESCape sequence to be read */
				}
			}
		}

		/* If chrout is nonzero, it contains next code to be sent to DCX board. */
		if (chrout)
		{
			/* Get present value at Data Input mailbox. If contents of mailbox
				 is nonzero, last byte has not been read by DCX. */
			if (getdcxbyt(DATAIN)==0)
			{
				putdcxbyt(DATAIN,chrout);		/* Write new data to DCX. */

				chrout = 0;
			}
		}

		/* Check for DCX sending data to Data Output mailbox. */
		if ((chrin=getdcxbyt(DATAOUT)) != 0)
		{
			putch(chrin);			/* Send char to screen for user display. */

			if (chrin == '\n')		/* Add carriage return to linefeeds*/
				putch('\r');

			/* Write a zero to Data In mail box to signal that we have
				received data and are ready for new data. */
			putdcxbyt(DATAOUT,0);
		}
	}
}


int getdcxbyt(loc)
	int loc;
{
	movedata(dcxseg,loc,FP_SEG(bytptr),FP_OFF(bytptr),1);
	return(byt);
}


putdcxbyt(loc,val)
	int loc,val;
{
	byt = val;
	movedata(FP_SEG(bytptr),FP_OFF(bytptr),dcxseg,loc,1);
}


/* Note:
    The movedata() function used in this program is available in the
    Microsoft C libraries.  It is used to move a specifed number of bytes
    from anywhere in the PC's memory to any other place.  It has the
    following format:

      movedata(src segment, src offset, dest segment, dest offset, count)

    If this program is being compiled under another brand, this function
    will have to be created or replaced with the equivalent.
*/
